@@ -76,7 +76,7 @@ callback (Gsasl * ctx, Gsasl_session * sctx, Gsasl_property prop)
 		|| (idp[n] >= 'A' && idp[n] <= 'Z')
 		|| (idp[n] >= '0' && idp[n] <= '9') || idp[n] == '.'))
 	    {
-	      printf ("Cannot handle identifier (%ld): %s\n",
+	      printf ("Cannot handle identifier (%zu): %s\n",
 		      (unsigned long) n, idp);
 	      return GSASL_AUTHENTICATION_ERROR;
 	    }
