@@ -33,6 +33,8 @@
 /* Get memcpy, strlen. */
 #include <string.h>
 
+#include "gc.h"
+
 /* Get tools. */
 #include "nonascii.h"
 #include "tokens.h"
@@ -63,7 +65,8 @@ struct _Gsasl_digest_md5_client_state
 typedef struct _Gsasl_digest_md5_client_state _Gsasl_digest_md5_client_state;
 
 int
-_gsasl_digest_md5_client_start (Gsasl_session * sctx, void **mech_data)
+_gsasl_digest_md5_client_start (Gsasl_session * sctx _GL_UNUSED,
+				void **mech_data)
 {
   _Gsasl_digest_md5_client_state *state;
   char nonce[CNONCE_ENTROPY_BYTES];
@@ -106,14 +109,15 @@ _gsasl_digest_md5_client_step (Gsasl_session * sctx,
   *output = NULL;
   *output_len = 0;
 
-  switch (state->step)
+  if (state->step == 0)
     {
-    case 0:
       state->step++;
       if (input_len == 0)
 	return GSASL_NEEDS_MORE;
-      /* fall through */
+    }
 
+  switch (state->step)
+    {
     case 1:
       {
 	if (digest_md5_parse_challenge (input, input_len,
@@ -144,11 +148,11 @@ _gsasl_digest_md5_client_step (Gsasl_session * sctx,
 	  const char *qop = gsasl_property_get (sctx, GSASL_QOP);
 
 	  if (!qop)
-	    state->response.qop = GSASL_QOP_AUTH;
+	    state->response.qop = DIGEST_MD5_QOP_AUTH;
 	  else if (strcmp (qop, "qop-int") == 0)
-	    state->response.qop = GSASL_QOP_AUTH_INT;
+	    state->response.qop = DIGEST_MD5_QOP_AUTH_INT;
 	  else if (strcmp (qop, "qop-auth") == 0)
-	    state->response.qop = GSASL_QOP_AUTH;
+	    state->response.qop = DIGEST_MD5_QOP_AUTH;
 	  else
 	    /* We don't support confidentiality or unknown
 	       keywords. */
@@ -213,11 +217,10 @@ _gsasl_digest_md5_client_step (Gsasl_session * sctx,
 	  if (rc < 0)
 	    return GSASL_MALLOC_ERROR;
 
-	  rc = _gsasl_hash (GSASL_HASH_MD5, tmp, strlen (tmp),
-			    state->secret);
+	  rc = gc_md5 (tmp, strlen (tmp), state->secret);
 	  free (tmp);
-	  if (rc != GSASL_OK)
-	    return rc;
+	  if (rc != GC_OK)
+	    return GSASL_CRYPTO_ERROR;
 	}
 
 	rc = digest_md5_hmac (state->response.response,
@@ -279,7 +282,8 @@ _gsasl_digest_md5_client_step (Gsasl_session * sctx,
 }
 
 void
-_gsasl_digest_md5_client_finish (Gsasl_session * sctx, void *mech_data)
+_gsasl_digest_md5_client_finish (Gsasl_session * sctx _GL_UNUSED,
+				 void *mech_data)
 {
   _Gsasl_digest_md5_client_state *state = mech_data;
 
@@ -294,7 +298,7 @@ _gsasl_digest_md5_client_finish (Gsasl_session * sctx, void *mech_data)
 }
 
 int
-_gsasl_digest_md5_client_encode (Gsasl_session * sctx,
+_gsasl_digest_md5_client_encode (Gsasl_session * sctx _GL_UNUSED,
 				 void *mech_data,
 				 const char *input,
 				 size_t input_len,
@@ -318,7 +322,7 @@ _gsasl_digest_md5_client_encode (Gsasl_session * sctx,
 }
 
 int
-_gsasl_digest_md5_client_decode (Gsasl_session * sctx,
+_gsasl_digest_md5_client_decode (Gsasl_session * sctx _GL_UNUSED,
 				 void *mech_data,
 				 const char *input,
 				 size_t input_len,
