@@ -33,6 +33,8 @@
 /* Get memcpy, strdup, strlen. */
 #include <string.h>
 
+#include "gc.h"
+
 /* Get tools. */
 #include "nonascii.h"
 #include "tokens.h"
@@ -64,7 +66,8 @@ struct _Gsasl_digest_md5_server_state
 typedef struct _Gsasl_digest_md5_server_state _Gsasl_digest_md5_server_state;
 
 int
-_gsasl_digest_md5_server_start (Gsasl_session * sctx, void **mech_data)
+_gsasl_digest_md5_server_start (Gsasl_session * sctx _GL_UNUSED,
+				void **mech_data)
 {
   _Gsasl_digest_md5_server_state *state;
   char nonce[NONCE_ENTROPY_BYTES];
@@ -274,11 +277,10 @@ _gsasl_digest_md5_server_step (Gsasl_session * sctx,
 	    if (rc < 0)
 	      return GSASL_MALLOC_ERROR;
 
-	    rc = _gsasl_hash (GSASL_HASH_MD5, tmp, strlen (tmp),
-			      state->secret);
+	    rc = gc_md5 (tmp, strlen (tmp), state->secret);
 	    free (tmp);
-	    if (rc != GSASL_OK)
-	      return rc;
+	    if (rc != GC_OK)
+	      return GSASL_CRYPTO_ERROR;
 	  }
 	else
 	  {
@@ -333,7 +335,8 @@ _gsasl_digest_md5_server_step (Gsasl_session * sctx,
 }
 
 void
-_gsasl_digest_md5_server_finish (Gsasl_session * sctx, void *mech_data)
+_gsasl_digest_md5_server_finish (Gsasl_session * sctx _GL_UNUSED,
+				 void *mech_data)
 {
   _Gsasl_digest_md5_server_state *state = mech_data;
 
@@ -348,7 +351,7 @@ _gsasl_digest_md5_server_finish (Gsasl_session * sctx, void *mech_data)
 }
 
 int
-_gsasl_digest_md5_server_encode (Gsasl_session * sctx,
+_gsasl_digest_md5_server_encode (Gsasl_session * sctx _GL_UNUSED,
 				 void *mech_data,
 				 const char *input,
 				 size_t input_len,
@@ -372,7 +375,7 @@ _gsasl_digest_md5_server_encode (Gsasl_session * sctx,
 }
 
 int
-_gsasl_digest_md5_server_decode (Gsasl_session * sctx,
+_gsasl_digest_md5_server_decode (Gsasl_session * sctx _GL_UNUSED,
 				 void *mech_data,
 				 const char *input,
 				 size_t input_len,
