@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004, 2007-2012 Free Software Foundation, Inc.
+ * Copyright (C) 2004, 2007-2020 Free Software Foundation, Inc.
  * Written by Bruno Haible and Eric Blake
  *
  * This program is free software: you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #include <config.h>
 
@@ -29,12 +29,6 @@ SIGNATURE_CHECK (memmem, void *, (void const *, size_t, void const *, size_t));
 #include "zerosize-ptr.h"
 #include "macros.h"
 
-static void *
-null_ptr (void)
-{
-  return NULL;
-}
-
 int
 main (int argc, char *argv[])
 {
@@ -43,8 +37,9 @@ main (int argc, char *argv[])
      caused by SIGALRM.  All known platforms that lack alarm also lack
      memmem, and the replacement memmem is known to not take too
      long.  */
+  int alarm_value = 100;
   signal (SIGALRM, SIG_DFL);
-  alarm (100);
+  alarm (alarm_value);
 #endif
 
   {
@@ -78,31 +73,35 @@ main (int argc, char *argv[])
   }
 
   /* Check that length 0 does not dereference the pointer.  */
-  {
-    const char *result = memmem (zerosize_ptr (), 0, "foo", 3);
-    ASSERT (result == NULL);
-  }
+  void *page_boundary = zerosize_ptr ();
+  if (page_boundary)
+    {
+      {
+        const char *result = memmem (page_boundary, 0, "foo", 3);
+        ASSERT (result == NULL);
+      }
 
-  {
-    const char input[] = "foo";
-    const char *result = memmem (input, strlen (input), null_ptr (), 0);
-    ASSERT (result == input);
-  }
+      {
+        const char input[] = "foo";
+        const char *result = memmem (input, strlen (input), page_boundary, 0);
+        ASSERT (result == input);
+      }
+    }
 
   /* Check that a long periodic needle does not cause false positives.  */
   {
-    const char input[] = ("F_BD_CE_BD_EF_BF_BD_EF_BF_BD_EF_BF_BD_EF_BF_BD"
-                          "_C3_88_20_EF_BF_BD_EF_BF_BD_EF_BF_BD"
-                          "_C3_A7_20_EF_BF_BD");
+    const char input[] = "F_BD_CE_BD_EF_BF_BD_EF_BF_BD_EF_BF_BD_EF_BF_BD"
+                         "_C3_88_20_EF_BF_BD_EF_BF_BD_EF_BF_BD"
+                         "_C3_A7_20_EF_BF_BD";
     const char need[] = "_EF_BF_BD_EF_BF_BD_EF_BF_BD_EF_BF_BD_EF_BF_BD";
     const char *result = memmem (input, strlen (input), need, strlen (need));
     ASSERT (result == NULL);
   }
   {
-    const char input[] = ("F_BD_CE_BD_EF_BF_BD_EF_BF_BD_EF_BF_BD_EF_BF_BD"
-                          "_C3_88_20_EF_BF_BD_EF_BF_BD_EF_BF_BD"
-                          "_C3_A7_20_EF_BF_BD_DA_B5_C2_A6_20"
-                          "_EF_BF_BD_EF_BF_BD_EF_BF_BD_EF_BF_BD_EF_BF_BD");
+    const char input[] = "F_BD_CE_BD_EF_BF_BD_EF_BF_BD_EF_BF_BD_EF_BF_BD"
+                         "_C3_88_20_EF_BF_BD_EF_BF_BD_EF_BF_BD"
+                         "_C3_A7_20_EF_BF_BD_DA_B5_C2_A6_20"
+                         "_EF_BF_BD_EF_BF_BD_EF_BF_BD_EF_BF_BD_EF_BF_BD";
     const char need[] = "_EF_BF_BD_EF_BF_BD_EF_BF_BD_EF_BF_BD_EF_BF_BD";
     const char *result = memmem (input, strlen (input), need, strlen (need));
     ASSERT (result == input + 115);
@@ -290,6 +289,29 @@ main (int argc, char *argv[])
         ASSERT (p);
         ASSERT (p - haystack == i);
       }
+    free (haystack);
+  }
+
+  /* Test long needles.  */
+  {
+    size_t m = 1024;
+    char *haystack = (char *) malloc (2 * m + 1);
+    char *needle = (char *) malloc (m + 1);
+    if (haystack != NULL && needle != NULL)
+      {
+        const char *p;
+        haystack[0] = 'x';
+        memset (haystack + 1, ' ', m - 1);
+        memset (haystack + m, 'x', m);
+        haystack[2 * m] = '\0';
+        memset (needle, 'x', m);
+        needle[m] = '\0';
+        p = memmem (haystack, strlen (haystack), needle, strlen (needle));
+        ASSERT (p);
+        ASSERT (p - haystack == m);
+      }
+    free (needle);
+    free (haystack);
   }
 
   return 0;
