@@ -33,6 +33,9 @@
 /* Get memcpy, strlen. */
 #include <string.h>
 
+/* Get FALLTHROUGH. */
+#include <attribute.h>
+
 #include "gss-extra.h"
 #include "gs2helper.h"
 #include "mechtools.h"
@@ -131,15 +134,9 @@ _gsasl_gs2_server_start (Gsasl_session * sctx, void **mech_data)
       return res;
     }
 
-  res = gs2_get_cred (sctx, state);
-  if (res != GSASL_OK)
-    {
-      free (state);
-      return res;
-    }
-
   state->step = 0;
   state->context = GSS_C_NO_CONTEXT;
+  state->cred = GSS_C_NO_CREDENTIAL;
   state->client = NULL;
   /* The initiator-address-type and acceptor-address-type fields of
      the GSS-CHANNEL-BINDINGS structure MUST be set to 0.  The
@@ -167,7 +164,7 @@ int
 _gsasl_gs2_server_step (Gsasl_session * sctx,
 			void *mech_data,
 			const char *input, size_t input_len,
-			char **output, size_t * output_len)
+			char **output, size_t *output_len)
 {
   _Gsasl_gs2_server_state *state = mech_data;
   gss_buffer_desc bufdesc1, bufdesc2;
@@ -180,19 +177,22 @@ _gsasl_gs2_server_step (Gsasl_session * sctx,
 
   *output = NULL;
   *output_len = 0;
-  bufdesc1.value = input;
+  bufdesc1.value = (char *) input;
   bufdesc1.length = input_len;
 
   switch (state->step)
     {
     case 0:
+      res = gs2_get_cred (sctx, state);
+      if (res != GSASL_OK)
+	return res;
       if (input_len == 0)
 	{
 	  res = GSASL_NEEDS_MORE;
 	  break;
 	}
       state->step++;
-      /* fall through */
+      FALLTHROUGH;		/* fall through */
 
     case 1:
       {
@@ -210,10 +210,10 @@ _gsasl_gs2_server_step (Gsasl_session * sctx,
 	    free (authzid);
 	  }
 
-	state->cb.application_data.value = input;
+	state->cb.application_data.value = (char *) input;
 	state->cb.application_data.length = headerlen;
 
-	bufdesc2.value = input + headerlen;
+	bufdesc2.value = (char *) input + headerlen;
 	bufdesc2.length = input_len - headerlen;
 
 	maj_stat = gss_encapsulate_token (&bufdesc2, state->mech_oid,
@@ -224,7 +224,7 @@ _gsasl_gs2_server_step (Gsasl_session * sctx,
 	free_bufdesc1 = 1;
       }
       state->step++;
-      /* fall through */
+      FALLTHROUGH;		/* fall through */
 
     case 2:
       if (state->client)
@@ -297,6 +297,7 @@ _gsasl_gs2_server_finish (Gsasl_session * sctx, void *mech_data)
 {
   _Gsasl_gs2_server_state *state = mech_data;
   OM_uint32 min_stat;
+  (void) sctx;
 
   if (!state)
     return;
