@@ -45,7 +45,7 @@ struct saml20_client_state
 };
 
 int
-_gsasl_saml20_client_start (Gsasl_session * sctx, void **mech_data)
+_gsasl_saml20_client_start (Gsasl_session * sctx _GL_UNUSED, void **mech_data)
 {
   struct saml20_client_state *state;
 
@@ -62,7 +62,7 @@ int
 _gsasl_saml20_client_step (Gsasl_session * sctx,
 			   void *mech_data,
 			   const char *input, size_t input_len,
-			   char **output, size_t * output_len)
+			   char **output, size_t *output_len)
 {
   struct saml20_client_state *state = mech_data;
   int res = GSASL_MECHANISM_CALLED_TOO_MANY_TIMES;
@@ -117,7 +117,7 @@ _gsasl_saml20_client_step (Gsasl_session * sctx,
 }
 
 void
-_gsasl_saml20_client_finish (Gsasl_session * sctx, void *mech_data)
+_gsasl_saml20_client_finish (Gsasl_session * sctx _GL_UNUSED, void *mech_data)
 {
   struct saml20_client_state *state = mech_data;
 
