@@ -41,6 +41,7 @@
 #include "printer.h"
 #include "gc.h"
 #include "memxor.h"
+#include "tools.h"
 #include "mechtools.h"
 
 #define CNONCE_ENTROPY_BYTES 18
@@ -62,12 +63,11 @@ struct scram_client_state
 };
 
 static int
-scram_start (Gsasl_session * sctx, void **mech_data,
-	     bool plus, Gsasl_hash hash)
+scram_start (Gsasl_session * sctx _GL_UNUSED,
+	     void **mech_data, bool plus, Gsasl_hash hash)
 {
   struct scram_client_state *state;
   char buf[CNONCE_ENTROPY_BYTES];
-  const char *p;
   int rc;
 
   state = (struct scram_client_state *) calloc (sizeof (*state), 1);
@@ -99,25 +99,6 @@ scram_start (Gsasl_session * sctx, void **mech_data,
     state->cf.client_nonce = strdup ("rOprNGfwEbeRWgbNEkqO");
 #endif
 
-  p = gsasl_property_get (sctx, GSASL_CB_TLS_UNIQUE);
-  if (state->plus && !p)
-    {
-      free (state->cf.client_nonce);
-      free (state);
-      return GSASL_NO_CB_TLS_UNIQUE;
-    }
-  if (p)
-    {
-      rc = gsasl_base64_from (p, strlen (p), &state->cbtlsunique,
-			      &state->cbtlsuniquelen);
-      if (rc != GSASL_OK)
-	{
-	  free (state->cf.client_nonce);
-	  free (state);
-	  return rc;
-	}
-    }
-
   *mech_data = state;
 
   return GSASL_OK;
@@ -170,6 +151,17 @@ _gsasl_scram_client_step (Gsasl_session * sctx,
       {
 	const char *p;
 
+	p = gsasl_property_get (sctx, GSASL_CB_TLS_UNIQUE);
+	if (state->plus && !p)
+	  return GSASL_NO_CB_TLS_UNIQUE;
+	if (p)
+	  {
+	    rc = gsasl_base64_from (p, strlen (p), &state->cbtlsunique,
+				    &state->cbtlsuniquelen);
+	    if (rc != GSASL_OK)
+	      return rc;
+	  }
+
 	if (state->plus)
 	  {
 	    state->cf.cbflag = 'p';
@@ -274,8 +266,6 @@ _gsasl_scram_client_step (Gsasl_session * sctx,
 	/* Generate ClientProof. */
 	{
 	  char saltedpassword[GSASL_HASH_MAX_SIZE];
-	  char clientsignature[GSASL_HASH_MAX_SIZE];
-	  char clientproof[GSASL_HASH_MAX_SIZE];
 	  char clientkey[GSASL_HASH_MAX_SIZE];
 	  char serverkey[GSASL_HASH_MAX_SIZE];
 	  char storedkey[GSASL_HASH_MAX_SIZE];
@@ -299,7 +289,6 @@ _gsasl_scram_client_step (Gsasl_session * sctx,
 	    }
 	  else if ((p = gsasl_property_get (sctx, GSASL_PASSWORD)) != NULL)
 	    {
-	      char hexstr_saltedpassword[GSASL_HASH_MAX_SIZE * 2 + 1];
 	      char *salt;
 	      size_t saltlen;
 
@@ -318,11 +307,8 @@ _gsasl_scram_client_step (Gsasl_session * sctx,
 	      if (rc != 0)
 		return rc;
 
-	      _gsasl_hex_encode (saltedpassword,
-				 gsasl_hash_length (state->hash),
-				 hexstr_saltedpassword);
-	      gsasl_property_set (sctx, GSASL_SCRAM_SALTED_PASSWORD,
-				  hexstr_saltedpassword);
+	      set_saltedpassword (sctx, state->hash, saltedpassword);
+
 	      gsasl_free (salt);
 	    }
 	  else
@@ -349,24 +335,30 @@ _gsasl_scram_client_step (Gsasl_session * sctx,
 	      return GSASL_MALLOC_ERROR;
 	  }
 
-	  /* ClientSignature := HMAC(StoredKey, AuthMessage) */
-	  rc = _gsasl_hmac (state->hash,
-			    storedkey,
-			    gsasl_hash_length (state->hash),
-			    state->authmessage,
-			    strlen (state->authmessage), clientsignature);
-	  if (rc != 0)
-	    return rc;
+	  {
+	    char clientsignature[GSASL_HASH_MAX_SIZE];
+	    char clientproof[GSASL_HASH_MAX_SIZE];
 
-	  /* ClientProof := ClientKey XOR ClientSignature */
-	  memcpy (clientproof, clientkey, gsasl_hash_length (state->hash));
-	  memxor (clientproof, clientsignature,
-		  gsasl_hash_length (state->hash));
+	    /* ClientSignature := HMAC(StoredKey, AuthMessage) */
+	    rc = _gsasl_hmac (state->hash,
+			      storedkey,
+			      gsasl_hash_length (state->hash),
+			      state->authmessage,
+			      strlen (state->authmessage), clientsignature);
+	    if (rc != 0)
+	      return rc;
 
-	  rc = gsasl_base64_to (clientproof, gsasl_hash_length (state->hash),
-				&state->cl.proof, NULL);
-	  if (rc != 0)
-	    return rc;
+	    /* ClientProof := ClientKey XOR ClientSignature */
+	    memcpy (clientproof, clientkey, gsasl_hash_length (state->hash));
+	    memxor (clientproof, clientsignature,
+		    gsasl_hash_length (state->hash));
+
+	    rc =
+	      gsasl_base64_to (clientproof, gsasl_hash_length (state->hash),
+			       &state->cl.proof, NULL);
+	    if (rc != 0)
+	      return rc;
+	  }
 
 	  /* Generate ServerSignature, for comparison in next step. */
 	  {
@@ -420,7 +412,7 @@ _gsasl_scram_client_step (Gsasl_session * sctx,
 }
 
 void
-_gsasl_scram_client_finish (Gsasl_session * sctx, void *mech_data)
+_gsasl_scram_client_finish (Gsasl_session * sctx _GL_UNUSED, void *mech_data)
 {
   struct scram_client_state *state = mech_data;
 
