@@ -44,6 +44,7 @@
 #include "printer.h"
 #include "gc.h"
 #include "memxor.h"
+#include "tools.h"
 #include "mechtools.h"
 
 #define DEFAULT_SALT_BYTES 12
@@ -51,8 +52,8 @@
 
 struct scram_server_state
 {
-  int plus;
-  int hash; // 0 is SHA1, 1 is SHA256
+  bool plus;
+  Gsasl_hash hash;
   int step;
   char *cbind;
   char *gs2header;		/* copy of client first gs2-header */
@@ -60,8 +61,8 @@ struct scram_server_state
   char *sf_str;			/* copy of server first message */
   char *snonce;
   char *clientproof;
-  char *storedkey;
-  char *serverkey;
+  char storedkey[GSASL_HASH_MAX_SIZE];
+  char serverkey[GSASL_HASH_MAX_SIZE];
   char *authmessage;
   char *cbtlsunique;
   size_t cbtlsuniquelen;
@@ -72,11 +73,11 @@ struct scram_server_state
 };
 
 static int
-scram_start (Gsasl_session * sctx, void **mech_data, int plus, int hash)
+scram_start (Gsasl_session * sctx _GL_UNUSED, void **mech_data,
+	     bool plus, Gsasl_hash hash)
 {
   struct scram_server_state *state;
   char buf[MAX (SNONCE_ENTROPY_BYTES, DEFAULT_SALT_BYTES)];
-  const char *p;
   int rc;
 
   state = (struct scram_server_state *) calloc (sizeof (*state), 1);
@@ -92,10 +93,10 @@ scram_start (Gsasl_session * sctx, void **mech_data, int plus, int hash)
 
   rc = gsasl_base64_to (buf, SNONCE_ENTROPY_BYTES, &state->snonce, NULL);
 #if SCRAMDEBUG
-  if (hash == 0)
-    state->snonce = strdup ("3rfcNHYJY1ZVvWVs7j"); // SCRAM-SHA1
+  if (hash == GSASL_HASH_SHA1)
+    state->snonce = strdup ("3rfcNHYJY1ZVvWVs7j");	// SCRAM-SHA1
   else
-    state->snonce = strdup ("%hvYDpWUa2RaTCAfuxFIlj)hNlF$k0"); // SCRAM-SHA256
+    state->snonce = strdup ("%hvYDpWUa2RaTCAfuxFIlj)hNlF$k0");	// SCRAM-SHA256
 #endif
   if (rc != GSASL_OK)
     goto end;
@@ -108,20 +109,6 @@ scram_start (Gsasl_session * sctx, void **mech_data, int plus, int hash)
   if (rc != GSASL_OK)
     goto end;
 
-  p = gsasl_property_get (sctx, GSASL_CB_TLS_UNIQUE);
-  if (plus && !p)
-    {
-      rc = GSASL_NO_CB_TLS_UNIQUE;
-      goto end;
-    }
-  if (p)
-    {
-      rc = gsasl_base64_from (p, strlen (p), &state->cbtlsunique,
-			      &state->cbtlsuniquelen);
-      if (rc != GSASL_OK)
-	goto end;
-    }
-
   *mech_data = state;
 
   return GSASL_OK;
@@ -137,13 +124,13 @@ end:
 int
 _gsasl_scram_sha1_server_start (Gsasl_session * sctx, void **mech_data)
 {
-  return scram_start (sctx, mech_data, 0, 0);
+  return scram_start (sctx, mech_data, false, GSASL_HASH_SHA1);
 }
 
 int
 _gsasl_scram_sha1_plus_server_start (Gsasl_session * sctx, void **mech_data)
 {
-  return scram_start (sctx, mech_data, 1, 0);
+  return scram_start (sctx, mech_data, true, GSASL_HASH_SHA1);
 }
 #endif
 
@@ -151,22 +138,46 @@ _gsasl_scram_sha1_plus_server_start (Gsasl_session * sctx, void **mech_data)
 int
 _gsasl_scram_sha256_server_start (Gsasl_session * sctx, void **mech_data)
 {
-  return scram_start (sctx, mech_data, 0, 1);
+  return scram_start (sctx, mech_data, false, GSASL_HASH_SHA256);
 }
 
 int
 _gsasl_scram_sha256_plus_server_start (Gsasl_session * sctx, void **mech_data)
 {
-  return scram_start (sctx, mech_data, 1, 1);
+  return scram_start (sctx, mech_data, true, GSASL_HASH_SHA256);
 }
 #endif
 
 static int
-scram_server_step (Gsasl_session * sctx,
-		   void *mech_data,
-		   const char *input,
-		   size_t input_len,
-		   char **output, size_t * output_len)
+extract_serverkey (struct scram_server_state *state,
+		   const char *b64, char *buf)
+{
+  char *bin;
+  size_t binlen;
+  int rc;
+
+  rc = gsasl_base64_from (b64, strlen (b64), &bin, &binlen);
+  if (rc != GSASL_OK)
+    return rc;
+
+  if (binlen != gsasl_hash_length (state->hash))
+    {
+      free (bin);
+      return GSASL_AUTHENTICATION_ERROR;
+    }
+
+  memcpy (buf, bin, binlen);
+
+  free (bin);
+
+  return GSASL_OK;
+}
+
+int
+_gsasl_scram_server_step (Gsasl_session * sctx,
+			  void *mech_data,
+			  const char *input,
+			  size_t input_len, char **output, size_t *output_len)
 {
   struct scram_server_state *state = mech_data;
   int res = GSASL_MECHANISM_CALLED_TOO_MANY_TIMES;
@@ -182,6 +193,21 @@ scram_server_step (Gsasl_session * sctx,
 	if (input_len == 0)
 	  return GSASL_NEEDS_MORE;
 
+	{
+	  const char *p;
+
+	  p = gsasl_property_get (sctx, GSASL_CB_TLS_UNIQUE);
+	  if (state->plus && !p)
+	    return GSASL_NO_CB_TLS_UNIQUE;
+	  if (p)
+	    {
+	      rc = gsasl_base64_from (p, strlen (p), &state->cbtlsunique,
+				      &state->cbtlsuniquelen);
+	      if (rc != GSASL_OK)
+		return rc;
+	    }
+	}
+
 	if (scram_parse_client_first (input, input_len, &state->cf) < 0)
 	  return GSASL_MECHANISM_PARSE_ERROR;
 
@@ -236,11 +262,6 @@ scram_server_step (Gsasl_session * sctx,
 	  size_t cnlen = strlen (state->cf.client_nonce);
 	  size_t snlen = strlen (state->snonce);
 
-#if SCRAMDEBUG
-	  printf ("Server.cnonce: %s\n", state->cf.client_nonce);
-	  printf ("Server.snonce: %s\n", state->snonce);
-#endif
-
 	  state->sf.nonce = malloc (cnlen + snlen + 1);
 	  if (!state->sf.nonce)
 	    return GSASL_MALLOC_ERROR;
@@ -248,9 +269,6 @@ scram_server_step (Gsasl_session * sctx,
 	  memcpy (state->sf.nonce, state->cf.client_nonce, cnlen);
 	  memcpy (state->sf.nonce + cnlen, state->snonce, snlen);
 	  state->sf.nonce[cnlen + snlen] = '\0';
-#if SCRAMDEBUG
-	  printf ("Server.nonce: %s\n", state->sf.nonce);
-#endif
 	}
 
 	gsasl_property_set (sctx, GSASL_AUTHID, state->cf.username);
@@ -258,10 +276,23 @@ scram_server_step (Gsasl_session * sctx,
 
 	{
 	  const char *p = gsasl_property_get (sctx, GSASL_SCRAM_ITER);
+
 	  if (p)
 	    state->sf.iter = strtoul (p, NULL, 10);
 	  if (!p || state->sf.iter == 0 || state->sf.iter == ULONG_MAX)
 	    state->sf.iter = 4096;
+
+	  /* Save salt/iter as properties, so that client callback can
+	     access them. */
+	  {
+	    char *str = NULL;
+	    int n;
+	    n = asprintf (&str, "%zu", state->sf.iter);
+	    if (n < 0 || str == NULL)
+	      return GSASL_MALLOC_ERROR;
+	    gsasl_property_set (sctx, GSASL_SCRAM_ITER, str);
+	    free (str);
+	  }
 	}
 
 	{
@@ -271,6 +302,8 @@ scram_server_step (Gsasl_session * sctx,
 	      free (state->sf.salt);
 	      state->sf.salt = strdup (p);
 	    }
+	  else
+	    gsasl_property_set (sctx, GSASL_SCRAM_SALT, state->sf.salt);
 	}
 
 	rc = scram_print_server_first (&state->sf, &state->sf_str);
@@ -332,7 +365,7 @@ scram_server_step (Gsasl_session * sctx,
 	}
 
 	/* Base64 decode client proof and check that length matches
-	   SHA1/SHA256 size. */
+	   hash size. */
 	{
 	  size_t len;
 
@@ -340,121 +373,67 @@ scram_server_step (Gsasl_session * sctx,
 				  &state->clientproof, &len);
 	  if (rc != 0)
 	    return rc;
-	  if ((state->hash == 0 && len != 20) || (state->hash == 1 && len != 32))
+	  if (gsasl_hash_length (state->hash) != len)
 	    return GSASL_MECHANISM_PARSE_ERROR;
 	}
 	{
-	  const char *p;
+	  const char *p, *q;
 
 	  /* Get StoredKey and ServerKey */
-	  if ((p = gsasl_property_get (sctx, GSASL_PASSWORD)))
+	  if ((p = gsasl_property_get (sctx, GSASL_SCRAM_SERVERKEY))
+	      && (q = gsasl_property_get (sctx, GSASL_SCRAM_STOREDKEY)))
 	    {
-	      Gc_rc err;
-	      char *salt;
-	      size_t saltlen;
-	      char saltedpassword[32];
-	      char *clientkey;
-	      char *preppasswd;
-
-	      rc = gsasl_saslprep (p, 0, &preppasswd, NULL);
+	      rc = extract_serverkey (state, p, state->serverkey);
 	      if (rc != GSASL_OK)
 		return rc;
+	      rc = extract_serverkey (state, q, state->storedkey);
+	      if (rc != GSASL_OK)
+		return rc;
+	    }
+	  else if ((p = gsasl_property_get (sctx, GSASL_PASSWORD)))
+	    {
+	      char *salt;
+	      size_t saltlen;
+	      char saltedpassword[GSASL_HASH_MAX_SIZE];
+	      char clientkey[GSASL_HASH_MAX_SIZE];
+	      char *b64str;
 
 	      rc = gsasl_base64_from (state->sf.salt, strlen (state->sf.salt),
 				      &salt, &saltlen);
+	      if (rc != GSASL_OK)
+		return rc;
+
+	      rc = gsasl_scram_secrets_from_password (state->hash,
+						      p,
+						      state->sf.iter,
+						      salt, saltlen,
+						      saltedpassword,
+						      clientkey,
+						      state->serverkey,
+						      state->storedkey);
+	      if (rc != GSASL_OK)
+		return rc;
+
+	      set_saltedpassword (sctx, state->hash, saltedpassword);
+
+	      rc = gsasl_base64_to (state->serverkey,
+				    gsasl_hash_length (state->hash),
+				    &b64str, NULL);
 	      if (rc != 0)
-		{
-		  gsasl_free (preppasswd);
-		  return rc;
-		}
+		return rc;
+	      gsasl_property_set (sctx, GSASL_SCRAM_SERVERKEY, b64str);
+	      free (b64str);
+
+
+	      rc = gsasl_base64_to (state->storedkey,
+				    gsasl_hash_length (state->hash),
+				    &b64str, NULL);
+	      if (rc != 0)
+		return rc;
+	      gsasl_property_set (sctx, GSASL_SCRAM_STOREDKEY, b64str);
+	      free (b64str);
 
-	      /* SaltedPassword := Hi(password, salt) */
-	      if (state->hash == 0)
-		err = gc_pbkdf2_sha1 (preppasswd, strlen (preppasswd),
-				      salt, saltlen,
-				      state->sf.iter, saltedpassword, 20);
-	      else
-		err = gc_pbkdf2_sha256 (preppasswd, strlen (preppasswd),
-				      salt, saltlen,
-				      state->sf.iter, saltedpassword, 32);
-	      gsasl_free (preppasswd);
 	      gsasl_free (salt);
-	      if (err != GC_OK)
-		return GSASL_MALLOC_ERROR;
-#if SCRAMDEBUG
-	      {
-		int i;
-		printf ("Server.SaltedPassword: ");
-		for (i = 0; i < (state->hash == 0 ? 20 : 32); i++)
-		  printf ("%02x", saltedpassword[i] & 0xFF);
-		printf ("\n");
-	      }
-#endif
-
-	      /* ClientKey := HMAC(SaltedPassword, "Client Key") */
-#define CLIENT_KEY "Client Key"
-	      if (state->hash == 0)
-		rc = gsasl_hmac_sha1 (saltedpassword, 20,
-				      CLIENT_KEY, strlen (CLIENT_KEY),
-				      &clientkey);
-	      else
-		rc = _gsasl_hmac_sha256 (saltedpassword, 32,
-					 CLIENT_KEY, strlen (CLIENT_KEY),
-					 &clientkey);
-	      if (rc != 0)
-		return rc;
-
-#if SCRAMDEBUG
-	      {
-		int i;
-		printf ("Server.ClientKey: ");
-		for (i = 0; i < (state->hash == 0 ? 20 : 32); i++)
-		  printf ("%02x", clientkey[i] & 0xFF);
-		printf ("\n");
-	      }
-#endif
-
-	      /* StoredKey := H(ClientKey) */
-	      if (state->hash == 0)
-		rc = gsasl_sha1 (clientkey, 20, &state->storedkey);
-	      else
-		rc = _gsasl_sha256 (clientkey, 32, &state->storedkey);
-	      free (clientkey);
-	      if (rc != 0)
-		return rc;
-
-#if SCRAMDEBUG
-	      {
-		int i;
-		printf ("Server.StoredKey: ");
-		for (i = 0; i < (state->hash == 0 ? 20 : 32); i++)
-		  printf ("%02x", state->storedkey[i] & 0xFF);
-		printf ("\n");
-	      }
-#endif
-
-	      /* ServerKey := HMAC(SaltedPassword, "Server Key") */
-#define SERVER_KEY "Server Key"
-	      if (state->hash == 0)
-		rc = gsasl_hmac_sha1 (saltedpassword, 20,
-				      SERVER_KEY, strlen (SERVER_KEY),
-				      &state->serverkey);
-	      else
-		rc = _gsasl_hmac_sha256 (saltedpassword, 32,
-					 SERVER_KEY, strlen (SERVER_KEY),
-					 &state->serverkey);
-	      if (rc != 0)
-		return rc;
-
-#if SCRAMDEBUG
-	      {
-		int i;
-		printf ("Server.ServerKey: ");
-		for (i = 0; i < (state->hash == 0 ? 20 : 32); i++)
-		  printf ("%02x", state->serverkey[i] & 0xFF);
-		printf ("\n");
-	      }
-#endif
 	    }
 	  else
 	    return GSASL_NO_PASSWORD;
@@ -477,117 +456,52 @@ scram_server_step (Gsasl_session * sctx,
 	    if (n <= 0 || !state->authmessage)
 	      return GSASL_MALLOC_ERROR;
 	  }
-#if SCRAMDEBUG
-	  printf ("Server.AuthMessage: %s\n", state->authmessage);
-#endif
 
 	  /* Check client proof. */
 	  {
-	    char *clientsignature;
-	    char *maybe_storedkey;
+	    char clientsignature[GSASL_HASH_MAX_SIZE];
+	    char maybe_storedkey[GSASL_HASH_MAX_SIZE];
 
 	    /* ClientSignature := HMAC(StoredKey, AuthMessage) */
-	    if (state->hash == 0)
-	      rc = gsasl_hmac_sha1 (state->storedkey, 20,
-				    state->authmessage,
-				    strlen (state->authmessage),
-				    &clientsignature);
-	    else
-	      rc = _gsasl_hmac_sha256 (state->storedkey, 32,
-				       state->authmessage,
-				       strlen (state->authmessage),
-				       &clientsignature);
+	    rc = _gsasl_hmac (state->hash,
+			      state->storedkey,
+			      gsasl_hash_length (state->hash),
+			      state->authmessage, strlen (state->authmessage),
+			      clientsignature);
 	    if (rc != 0)
 	      return rc;
 
-#if SCRAMDEBUG
-	    {
-	      int i;
-	      printf ("Server.ClientSignature: ");
-	      for (i = 0; i < (state->hash == 0 ? 20 : 32); i++)
-		printf ("%02x", clientsignature[i] & 0xFF);
-	      printf ("\n");
-	    }
-#endif
-
 	    /* ClientKey := ClientProof XOR ClientSignature */
-	    if (state->hash == 0)
-	      memxor (clientsignature, state->clientproof, 20);
-	    else
-	      memxor (clientsignature, state->clientproof, 32);
+	    memxor (clientsignature, state->clientproof,
+		    gsasl_hash_length (state->hash));
 
-#if SCRAMDEBUG
-	    {
-	      int i;
-	      printf ("Server.ClientKey: ");
-	      for (i = 0; i < (state->hash == 0 ? 20 : 32); i++)
-		printf ("%02x", clientsignature[i] & 0xFF);
-	      printf ("\n");
-	    }
-#endif
-
-	    if (state->hash == 0)
-	      rc = gsasl_sha1 (clientsignature, 20, &maybe_storedkey);
-	    else
-	      rc = _gsasl_sha256 (clientsignature, 32, &maybe_storedkey);
-	    free (clientsignature);
+	    rc = _gsasl_hash (state->hash, clientsignature,
+			      gsasl_hash_length (state->hash),
+			      maybe_storedkey);
 	    if (rc != 0)
 	      return rc;
 
-#if SCRAMDEBUG
-	    {
-	      int i;
-	      printf ("Server.StoredKey: ");
-	      for (i = 0; i < (state->hash == 0 ? 20 : 32); i++)
-		printf ("%02x", maybe_storedkey[i] & 0xFF);
-	      printf ("\n");
-	    }
-#endif
-
-	    if (state->hash == 0)
-	      rc = memcmp (state->storedkey, maybe_storedkey, 20);
-	    else
-	      rc = memcmp (state->storedkey, maybe_storedkey, 32);
-	    free (maybe_storedkey);
+	    rc = memcmp (state->storedkey, maybe_storedkey,
+			 gsasl_hash_length (state->hash));
 	    if (rc != 0)
 	      return GSASL_AUTHENTICATION_ERROR;
 	  }
 
 	  /* Generate server verifier. */
 	  {
-	    char *serversignature;
+	    char serversignature[GSASL_HASH_MAX_SIZE];
 
 	    /* ServerSignature := HMAC(ServerKey, AuthMessage) */
-	    if (state->hash == 0)
-	      rc = gsasl_hmac_sha1 (state->serverkey, 20,
-				  state->authmessage,
-				  strlen (state->authmessage),
-				  &serversignature);
-	    else
-	      rc = _gsasl_hmac_sha256 (state->serverkey, 32,
-				       state->authmessage,
-				       strlen (state->authmessage),
-				       &serversignature);
+	    rc = _gsasl_hmac (state->hash, state->serverkey,
+			      gsasl_hash_length (state->hash),
+			      state->authmessage,
+			      strlen (state->authmessage), serversignature);
 	    if (rc != 0)
 	      return rc;
 
-#if SCRAMDEBUG
-	    {
-	      int i;
-	      printf ("Server.ServerSignature: ");
-	      for (i = 0; i < (state->hash == 0 ? 20 : 32); i++)
-		printf ("%02x", serversignature[i] & 0xFF);
-	      printf ("\n");
-	    }
-#endif
-
-	    if (state->hash == 0)
-	      rc = gsasl_base64_to (serversignature, 20,
+	    rc = gsasl_base64_to (serversignature,
+				  gsasl_hash_length (state->hash),
 				  &state->sl.verifier, NULL);
-	    else
-	      rc = gsasl_base64_to (serversignature, 32,
-				    &state->sl.verifier, NULL);
-	    free (serversignature);
 	    if (rc != 0)
 	      return rc;
 	  }
@@ -610,32 +524,8 @@ scram_server_step (Gsasl_session * sctx,
   return res;
 }
 
-#ifdef USE_SCRAM_SHA1
-int
-_gsasl_scram_sha1_server_step (Gsasl_session * sctx,
-			       void *mech_data,
-			       const char *input,
-			       size_t input_len,
-			       char **output, size_t * output_len)
-{
-  return scram_server_step (sctx, mech_data, input, input_len, output, output_len);
-}
-#endif
-
-#ifdef USE_SCRAM_SHA256
-int
-_gsasl_scram_sha256_server_step (Gsasl_session * sctx,
-				 void *mech_data,
-				 const char *input,
-				 size_t input_len,
-				 char **output, size_t * output_len)
-{
-  return scram_server_step (sctx, mech_data, input, input_len, output, output_len);
-}
-#endif
-
-static void
-scram_server_finish (Gsasl_session * sctx, void *mech_data)
+void
+_gsasl_scram_server_finish (Gsasl_session * sctx _GL_UNUSED, void *mech_data)
 {
   struct scram_server_state *state = mech_data;
 
@@ -648,8 +538,6 @@ scram_server_finish (Gsasl_session * sctx, void *mech_data)
   free (state->sf_str);
   free (state->snonce);
   free (state->clientproof);
-  free (state->storedkey);
-  free (state->serverkey);
   free (state->authmessage);
   free (state->cbtlsunique);
   scram_free_client_first (&state->cf);
@@ -659,19 +547,3 @@ scram_server_finish (Gsasl_session * sctx, void *mech_data)
 
   free (state);
 }
-
-#ifdef USE_SCRAM_SHA1
-void
-_gsasl_scram_sha1_server_finish (Gsasl_session * sctx, void *mech_data)
-{
-  return scram_server_finish (sctx, mech_data);
-}
-#endif
-
-#ifdef USE_SCRAM_SHA256
-void
-_gsasl_scram_sha256_server_finish (Gsasl_session * sctx, void *mech_data)
-{
-  return scram_server_finish (sctx, mech_data);
-}
-#endif
