@@ -97,7 +97,7 @@ unescape_authzid (const char *str, size_t len, char **authzid)
    code.*/
 int
 _gsasl_parse_gs2_header (const char *data, size_t len,
-			 char **authzid, size_t * headerlen)
+			 char **authzid, size_t *headerlen)
 {
   char *authzid_endptr;
 
@@ -171,7 +171,7 @@ int
 _gsasl_gs2_generate_header (bool nonstd, char cbflag,
 			    const char *cbname, const char *authzid,
 			    size_t extralen, const char *extra,
-			    char **gs2h, size_t * gs2hlen)
+			    char **gs2h, size_t *gs2hlen)
 {
   int elen = extralen;
   char *gs2cbflag;
@@ -219,49 +219,188 @@ _gsasl_gs2_generate_header (bool nonstd, char cbflag,
   return GSASL_OK;
 }
 
-/*
- * _gsasl_sha256:
- * @in: input character array of data to hash.
- * @inlen: length of input character array of data to hash.
- * @out: newly allocated 32-byte character array with hash of data.
- *
- * Compute hash of data using SHA256.  The @out buffer must be
- * deallocated by the caller.
- *
- * Return value: Returns %GSASL_OK iff successful.
- *
- * Since: 1.9.0
- **/
-int
-_gsasl_sha256 (const char *in, size_t inlen, char *out[])
+/* Hex encode binary octet array IN of INLEN length, putting the hex
+   encoded string in OUT which must have room for the data and
+   terminating zero, i.e., 2*INLEN+1. */
+void
+_gsasl_hex_encode (const char *in, size_t inlen, char *out)
 {
-  *out = malloc (GC_SHA256_DIGEST_SIZE);
-  if (!*out)
-    return GSASL_MALLOC_ERROR;
-  return gc_sha256 (in, inlen, *out);
+  size_t i;
+  const char *p = in;
+
+  for (i = 0; i < 2 * inlen;)
+    {
+      unsigned char c = *p++;
+      out[i++] = "0123456789abcdef"[c >> 4];
+      out[i++] = "0123456789abcdef"[c & 0x0f];
+    }
+
+  out[i] = '\0';
+}
+
+static char
+hexdigit_to_char (char hexdigit)
+{
+  if (hexdigit >= '0' && hexdigit <= '9')
+    return hexdigit - '0';
+  if (hexdigit >= 'a' && hexdigit <= 'f')
+    return hexdigit - 'a' + 10;
+  return 0;
+}
+
+static char
+hex_to_char (char u, char l)
+{
+  return (char) (((unsigned char) hexdigit_to_char (u)) * 16
+		 + hexdigit_to_char (l));
+}
+
+/* Hex decode string HEXSTR containing only hex "0-9A-F" characters
+   into binary buffer BIN which must have room for data, i.e., strlen
+   (hexstr)/2. */
+void
+_gsasl_hex_decode (const char *hexstr, char *bin)
+{
+  while (*hexstr)
+    {
+      *bin = hex_to_char (hexstr[0], hexstr[1]);
+      hexstr += 2;
+      bin++;
+    }
+}
+
+/* Return whether string contains hex "0-9a-f" symbols only. */
+bool
+_gsasl_hex_p (const char *hexstr)
+{
+  static const char hexalpha[] = "0123456789abcdef";
+
+  for (; *hexstr; hexstr++)
+    if (strchr (hexalpha, *hexstr) == NULL)
+      return false;
+
+  return true;
 }
 
 /*
- * _gsasl_hmac_sha256:
+ * _gsasl_hash:
+ * @hash: a %Gsasl_hash hash algorithm identifier, e.g. #GSASL_HASH_SHA256.
+ * @in: input character array of data to hash.
+ * @inlen: length of input character array of data to hash.
+ * @outhash: buffer to hold hash of data.
+ *
+ * Compute hash of data using the @hash algorithm.  The @outhash
+ * buffer must have room to hold the size of @hash's output; a safe
+ * value that have room for all possible outputs is
+ * %GSASL_HASH_MAX_SIZE.
+ *
+ * Return value: Returns %GSASL_OK iff successful.
+ *
+ * Since: 1.10
+ **/
+int
+_gsasl_hash (Gsasl_hash hash, const char *in, size_t inlen, char *outhash)
+{
+  int rc;
+
+  if (hash == GSASL_HASH_SHA1)
+    rc = gc_sha1 (in, inlen, outhash);
+  else if (hash == GSASL_HASH_SHA256)
+    rc = gc_sha256 (in, inlen, outhash);
+  else
+    rc = GSASL_CRYPTO_ERROR;
+
+  return rc;
+}
+
+/*
+ * _gsasl_hmac:
+ * @hash: a %Gsasl_hash hash algorithm identifier, e.g. #GSASL_HASH_SHA256.
  * @key: input character array with key to use.
  * @keylen: length of input character array with key to use.
  * @in: input character array of data to hash.
  * @inlen: length of input character array of data to hash.
- * @outhash: newly allocated 32-byte character array with keyed hash of data.
+ * @outhash: buffer to hold keyed hash of data.
  *
- * Compute keyed checksum of data using HMAC-SHA256.  The @outhash buffer
- * must be deallocated by the caller.
+ * Compute keyed checksum of data using HMAC for the @hash algorithm.
+ * The @outhash buffer must have room to hold the size of @hash's
+ * output; a safe value that have room for all possible outputs is
+ * %GSASL_HASH_MAX_SIZE.
  *
  * Return value: Returns %GSASL_OK iff successful.
  *
- * Since: 1.9.0
+ * Since: 1.10
  **/
 int
-_gsasl_hmac_sha256 (const char *key, size_t keylen,
-		    const char *in, size_t inlen, char *outhash[])
+_gsasl_hmac (Gsasl_hash hash,
+	     const char *key, size_t keylen,
+	     const char *in, size_t inlen, char *outhash)
 {
-  *outhash = malloc (GC_SHA256_DIGEST_SIZE);
-  if (!*outhash)
-    return GSASL_MALLOC_ERROR;
-  return gc_hmac_sha256 (key, keylen, in, inlen, *outhash);
+  int rc;
+
+  if (hash == GSASL_HASH_SHA1)
+    rc = gc_hmac_sha1 (key, keylen, in, inlen, outhash);
+  else if (hash == GSASL_HASH_SHA256)
+    rc = gc_hmac_sha256 (key, keylen, in, inlen, outhash);
+  else
+    rc = GSASL_CRYPTO_ERROR;
+
+  return rc;
+}
+
+/*
+ * gsasl_pbkdf2:
+ * @hash: a %Gsasl_hash hash algorithm identifier.
+ * @password: input character array with password to use.
+ * @passwordlen: length of @password.
+ * @salt: input character array with salt, typically a short string.
+ * @saltlen: length of @salt.
+ * @c: iteration count, typically larger than 4096.
+ * @dk: output buffer, must be able to hold @dklen.
+ * @dklen: length of output buffer, or 0 to indicate @hash output size.
+ *
+ * Hash and salt password according to PBKDF2 algorithm with the @hash
+ * function used in HMAC.  This function can be used to prepare SCRAM
+ * SaltedPassword values for the %GSASL_SCRAM_SALTED_PASSWORD
+ * property.  Note that password should normally be prepared using
+ * gsasl_saslprep(GSASL_ALLOW_UNASSIGNED) before calling this
+ * function.
+ *
+ * Return value: Returns %GSASL_OK if successful, or error code.
+ *
+ * Since: 1.10
+ **/
+int
+_gsasl_pbkdf2 (Gsasl_hash hash,
+	       const char *password, size_t passwordlen,
+	       const char *salt, size_t saltlen,
+	       unsigned int c, char *dk, size_t dklen)
+{
+  int rc;
+  Gc_hash gch;
+
+  switch (hash)
+    {
+    case GSASL_HASH_SHA1:
+      if (dklen == 0)
+	dklen = GSASL_HASH_SHA1_SIZE;
+      gch = GC_SHA1;
+      break;
+
+    case GSASL_HASH_SHA256:
+      if (dklen == 0)
+	dklen = GSASL_HASH_SHA256_SIZE;
+      gch = GC_SHA256;
+      break;
+
+    default:
+      return GSASL_CRYPTO_ERROR;
+    }
+
+  rc = gc_pbkdf2_hmac (gch, password, passwordlen,
+		       salt, saltlen, c, dk, dklen);
+  if (rc != GC_OK)
+    return GSASL_CRYPTO_ERROR;
+
+  return GSASL_OK;
 }
