@@ -29,9 +29,9 @@ main (void)
   size_t i;
 
   for (i = 0; i < (sizeof (errors) / sizeof (errors[0])); i++)
-    if (errors[i].rc != i)
+    if (errors[i].rc != (int) i)
       {
-	printf ("%ld\n", i);
+	printf ("%i\n", errors[i].rc);
 	return EXIT_FAILURE;
       }
 
