@@ -27,7 +27,8 @@
 
 #ifdef HAVE_LIBGNUTLS
 #include <gnutls/gnutls.h>
-gnutls_session session;
+#include <gnutls/x509.h>
+gnutls_session_t session;
 bool using_tls = false;
 #endif
 
@@ -36,6 +37,104 @@ char *b64cbtlsunique = NULL;
 struct gengetopt_args_info args_info;
 int sockfd = 0;
 
+#ifdef HAVE_LIBGNUTLS
+static bool
+_handle_tlserror (int error)
+{
+  int rc;
+
+  switch (error)
+    {
+    case GNUTLS_E_REHANDSHAKE:
+      for (;;)
+	{
+	  rc = gnutls_handshake (session);
+	  switch (rc)
+	    {
+	    case GNUTLS_E_INTERRUPTED:
+	    case GNUTLS_E_AGAIN:
+	      continue;
+
+	    case GNUTLS_E_GOT_APPLICATION_DATA:
+	      /* TODO: signal this somehow? */
+	      continue;
+
+	    case GNUTLS_E_WARNING_ALERT_RECEIVED:
+	      fprintf (stderr, "ALERT: %s\n",
+		       gnutls_alert_get_name (gnutls_alert_get (session)));
+	      continue;
+
+	    default:
+	      fprintf (stderr, "TLS rehandshake failed: %s\n",
+		       gnutls_strerror (rc));
+	      /* make every error fatal */
+	      return false;
+	    }
+
+	  return true;
+	}
+
+    case GNUTLS_E_INTERRUPTED:
+    case GNUTLS_E_AGAIN:
+      /* not fatal */
+      return true;
+
+    default:
+      fprintf (stderr, "TLS error: %s\n", gnutls_strerror (error));
+      return false;
+    }
+}
+#endif
+
+static ssize_t
+_recv (void *dst, size_t cnt)
+{
+#ifdef HAVE_LIBGNUTLS
+  if (using_tls)
+    {
+      ssize_t l = 0;
+      do
+	{
+	  l = gnutls_record_recv (session, dst, cnt);
+
+	  if (l < 0 && !_handle_tlserror (l))
+	    break;
+	}
+      while (l < 0);
+
+      return l;
+    }
+#endif
+
+  return recv (sockfd, dst, cnt, 0);
+}
+
+static ssize_t
+_send (void const *src, size_t cnt)
+{
+#ifdef HAVE_LIBGNUTLS
+  if (using_tls)
+    {
+      ssize_t l;
+      do
+	{
+	  if (cnt > 0)
+	    l = gnutls_record_send (session, src, cnt);
+	  else
+	    l = 0;
+
+	  if (l < 0 && !_handle_tlserror (l))
+	    break;
+	}
+      while (l < 0);
+
+      return l;
+    }
+#endif
+
+  return write (sockfd, src, cnt);
+}
+
 int
 writeln (const char *str)
 {
@@ -45,30 +144,13 @@ writeln (const char *str)
     {
       ssize_t len = strlen (str);
 
-#ifdef HAVE_LIBGNUTLS
-      if (using_tls)
-	{
-	  /* GnuTLS < 1.2.9 cannot handle data != NULL && count == 0,
-	     it will return an error. */
-	  if (len > 0)
-	    len = gnutls_record_send (session, str, len);
-	  else
-	    len = 0;
-	}
-      else
-#endif
-	len = write (sockfd, str, len);
-      if (len != strlen (str))
+      len = _send (str, len);
+      if (len != (ssize_t) strlen (str))
 	return 0;
 
 #define CRLF "\r\n"
 
-#ifdef HAVE_LIBGNUTLS
-      if (using_tls)
-	len = gnutls_record_send (session, CRLF, strlen (CRLF));
-      else
-#endif
-	len = write (sockfd, CRLF, strlen (CRLF));
+      len = _send (CRLF, strlen (CRLF));
       if (len != strlen (CRLF))
 	return 0;
     }
@@ -93,12 +175,7 @@ readln (char **out)
 	  if (used == allocated)
 	    input = x2realloc (input, &allocated);
 
-#ifdef HAVE_LIBGNUTLS
-	  if (using_tls)
-	    nread = gnutls_record_recv (session, &input[used], 1);
-	  else
-#endif
-	    nread = recv (sockfd, &input[used], 1, 0);
+	  nread = _recv (&input[used], 1);
 	  if (nread <= 0)
 	    return 0;
 
@@ -367,9 +444,12 @@ mkpasswd (void)
   if (res != GSASL_OK)
     error (EXIT_FAILURE, 0, "%s", gsasl_strerror (res));
 
-  printf ("%s:%d:%s:%s:%s:%s\n", args_info.mechanism_arg,
+  printf ("{%s}%d,%s,%s,%s", args_info.mechanism_arg,
 	  args_info.iteration_count_arg,
-	  b64salt, hexsaltedpassword, b64serverkey, b64storedkey);
+	  b64salt, b64storedkey, b64serverkey);
+  if (args_info.verbose_given)
+    printf (",%s", hexsaltedpassword);
+  printf ("\n");
 
   if (salt != salt_buf)
     free (salt);
@@ -378,7 +458,6 @@ mkpasswd (void)
   free (b64serverkey);
   free (b64storedkey);
   free (hexsaltedpassword);
-  free (b64salt);
 }
 
 
@@ -391,8 +470,8 @@ main (int argc, char *argv[])
   char *connect_hostname = NULL;
   char *connect_service = NULL;
 #ifdef HAVE_LIBGNUTLS
-  gnutls_anon_client_credentials anoncred;
-  gnutls_certificate_credentials x509cred;
+  gnutls_anon_client_credentials_t anoncred;
+  gnutls_certificate_credentials_t x509cred;
 #endif
 
   set_program_name (argv[0]);
@@ -621,6 +700,13 @@ main (int argc, char *argv[])
 	error (EXIT_FAILURE, 0, _("setting GnuTLS defaults failed: %s"),
 	       gnutls_strerror (res));
 
+      res =
+	gnutls_server_name_set (session, GNUTLS_NAME_DNS, connect_hostname,
+				strlen (connect_hostname));
+      if (res < 0)
+	error (EXIT_FAILURE, 0, _("setting GnuTLS server name failed: %s"),
+	       gnutls_strerror (res));
+
       res = gnutls_anon_allocate_client_credentials (&anoncred);
       if (res < 0)
 	error (EXIT_FAILURE, 0,
@@ -645,7 +731,7 @@ main (int argc, char *argv[])
 	error (EXIT_FAILURE, 0, _("loading X.509 GnuTLS credential: %s"),
 	       gnutls_strerror (res));
 
-      if (args_info.x509_ca_file_arg)
+      if (args_info.x509_ca_file_arg && *args_info.x509_ca_file_arg)
 	{
 	  res = gnutls_certificate_set_x509_trust_file
 	    (x509cred, args_info.x509_ca_file_arg, GNUTLS_X509_FMT_PEM);
@@ -655,6 +741,13 @@ main (int argc, char *argv[])
 	  if (res == 0)
 	    error (EXIT_FAILURE, 0, _("no X.509 CAs found"));
 	}
+      else if (!args_info.x509_ca_file_arg)
+	{
+	  res = gnutls_certificate_set_x509_system_trust (x509cred);
+	  if (res < 0)
+	    error (EXIT_FAILURE, 0, _("setting GnuTLS system trust: %s"),
+		   gnutls_strerror (res));
+	}
 
       res = gnutls_credentials_set (session, GNUTLS_CRD_CERTIFICATE,
 				    x509cred);
@@ -662,6 +755,10 @@ main (int argc, char *argv[])
 	error (EXIT_FAILURE, 0, _("setting X.509 GnuTLS credential: %s"),
 	       gnutls_strerror (res));
 
+      if (args_info.x509_ca_file_arg == NULL
+	  || *args_info.x509_ca_file_arg)
+	gnutls_session_set_verify_cert (session, connect_hostname, 0);
+
       if (args_info.priority_arg)
 	{
 	  const char *err_pos;
@@ -674,18 +771,74 @@ main (int argc, char *argv[])
 		   gnutls_strerror (res), err_pos);
 	}
 
-      gnutls_transport_set_ptr (session, (gnutls_transport_ptr)
+      gnutls_transport_set_ptr (session, (gnutls_transport_ptr_t)
 				(unsigned long) sockfd);
 
       if (!starttls ())
 	return 1;
 
-      res = gnutls_handshake (session);
+      do
+	{
+	  res = gnutls_handshake (session);
+	}
+      while (res < 0 && gnutls_error_is_fatal (res) == 0);
+
+      if (!args_info.quiet_given)
+	{
+	int type;
+	unsigned status;
+	gnutls_datum_t out;
+
+	type = gnutls_certificate_type_get (session);
+	status = gnutls_session_get_verify_cert_status (session);
+	gnutls_certificate_verification_status_print (status, type, &out, 0);
+	fprintf (stderr, _("TLS X.509 Verification: %s\n"), out.data);
+	gnutls_free (out.data);
+      }
+
       if (res < 0)
 	error (EXIT_FAILURE, 0, _("GnuTLS handshake failed: %s"),
 	       gnutls_strerror (res));
 
-      if (args_info.x509_ca_file_arg)
+      if (args_info.verbose_given)
+	{
+	  char *desc = gnutls_session_get_desc (session);
+	  const gnutls_datum_t *cert_list;
+	  unsigned int cert_list_size = 0, i;
+	  gnutls_x509_crt_t cert;
+	  gnutls_datum_t out;
+
+	  fprintf (stderr, _("TLS session info: %s\n"), desc);
+	  gnutls_free (desc);
+	  fflush (stderr);
+
+	  cert_list = gnutls_certificate_get_peers (session, &cert_list_size);
+
+	  for (i = 0; i < cert_list_size; i++)
+	    {
+	      res = gnutls_x509_crt_init (&cert);
+	      if (res < 0)
+		continue;
+
+	      res = gnutls_x509_crt_import (cert, &cert_list[i],
+					    GNUTLS_X509_FMT_DER);
+	      if (res < 0)
+		continue;
+
+	      res = gnutls_x509_crt_print (cert, GNUTLS_CRT_PRINT_ONELINE,
+					   &out);
+	      if (res == 0)
+		{
+		  fprintf (stderr, _("TLS X.509 Certificate %u: %s\n"), i,
+			   out.data);
+		  gnutls_free (out.data);
+		}
+
+	      gnutls_x509_crt_deinit (cert);
+	    }
+	}
+
+      if (args_info.x509_ca_file_arg && *args_info.x509_ca_file_arg)
 	{
 	  unsigned int status;
 
@@ -712,7 +865,7 @@ main (int argc, char *argv[])
 #if HAVE_GNUTLS_SESSION_CHANNEL_BINDING
       if (!args_info.no_cb_flag)
 	{
-	  gnutls_datum cb;
+	  gnutls_datum_t cb;
 
 	  res = gnutls_session_channel_binding (session,
 						GNUTLS_CB_TLS_UNIQUE, &cb);
@@ -900,13 +1053,8 @@ main (int argc, char *argv[])
 
 		  if (sockfd)
 		    {
-#ifdef HAVE_LIBGNUTLS
-		      if (using_tls)
-			len = gnutls_record_send (session, out, output_len);
-		      else
-#endif
-			len = write (sockfd, out, output_len);
-		      if (len != output_len)
+		      len = _send (out, output_len);
+		      if (len != (ssize_t) output_len)
 			error (EXIT_FAILURE, errno, "write");
 		    }
 		  else if (!(strlen (line) == output_len &&
@@ -944,14 +1092,7 @@ main (int argc, char *argv[])
 		    sockbuf = x2realloc (sockbuf, &sockalloc1);
 		  sockalloc = sockalloc1;
 
-#ifdef HAVE_LIBGNUTLS
-		  if (using_tls)
-		    len = gnutls_record_recv (session, &sockbuf[sockpos],
-					      sockalloc - sockpos);
-		  else
-#endif
-		    len = recv (sockfd, &sockbuf[sockpos],
-				sockalloc - sockpos, 0);
+		  len = _recv (&sockbuf[sockpos], sockalloc - sockpos);
 		  if (len <= 0)
 		    break;
 
