@@ -52,9 +52,12 @@ callback (Gsasl * ctx, Gsasl_session * sctx, Gsasl_property prop)
       rc = GSASL_OK;
       break;
 
+      /* SCRAM/SALT: Attempt to reproduce RFC 7677 test vector.
+         Requires that SCRAMDEBUG=1 when compiling so that
+         client/server nonce ends up the same as in the document.  */
+
     case GSASL_SCRAM_ITER:
-      if (strcmp (gsasl_property_fast (sctx, GSASL_AUTHID),
-		  USERNAME) != 0)
+      if (strcmp (gsasl_property_fast (sctx, GSASL_AUTHID), USERNAME) != 0)
 	fail ("Username mismatch: %s",
 	      gsasl_property_fast (sctx, GSASL_AUTHID));
       gsasl_property_set (sctx, prop, "4096");
@@ -69,6 +72,8 @@ callback (Gsasl * ctx, Gsasl_session * sctx, Gsasl_property prop)
     case GSASL_CB_TLS_UNIQUE:
     case GSASL_AUTHZID:
     case GSASL_SCRAM_SALTED_PASSWORD:
+    case GSASL_SCRAM_SERVERKEY:
+    case GSASL_SCRAM_STOREDKEY:
       break;
 
     default:
@@ -128,29 +133,25 @@ doit (void)
   res = gsasl_step (client, s1, s1len, &s1, &s1len);
   if (res != GSASL_NEEDS_MORE)
     {
-      fail ("gsasl_step(1) failed (%d):\n%s\n", res,
-	    gsasl_strerror (res));
+      fail ("gsasl_step(1) failed (%d):\n%s\n", res, gsasl_strerror (res));
       return;
     }
 
   if (debug)
-    printf ("C: %.*s [%c]\n", (int) s1len,
-	    s1, res == GSASL_OK ? 'O' : 'N');
+    printf ("C: %.*s [%c]\n", (int) s1len, s1, res == GSASL_OK ? 'O' : 'N');
 
-  /* Server step... */
+  /* Server first... */
 
   res = gsasl_step (server, s1, s1len, &s2, &s2len);
   gsasl_free (s1);
   if (res != GSASL_NEEDS_MORE)
     {
-      fail ("gsasl_step(2) failed (%d):\n%s\n", res,
-	    gsasl_strerror (res));
+      fail ("gsasl_step(2) failed (%d):\n%s\n", res, gsasl_strerror (res));
       return;
     }
 
   if (debug)
-    printf ("S: %.*s [%c]\n", (int) s2len,
-	    s2, res == GSASL_OK ? 'O' : 'N');
+    printf ("S: %.*s [%c]\n", (int) s2len, s2, res == GSASL_OK ? 'O' : 'N');
 
   /* Client final... */
 
@@ -158,14 +159,12 @@ doit (void)
   gsasl_free (s2);
   if (res != GSASL_NEEDS_MORE)
     {
-      fail ("gsasl_step(3) failed (%d):\n%s\n", res,
-	    gsasl_strerror (res));
+      fail ("gsasl_step(3) failed (%d):\n%s\n", res, gsasl_strerror (res));
       return;
     }
 
   if (debug)
-    printf ("C: %.*s [%c]\n", (int) s1len,
-	    s1, res == GSASL_OK ? 'O' : 'N');
+    printf ("C: %.*s [%c]\n", (int) s1len, s1, res == GSASL_OK ? 'O' : 'N');
 
   /* Server final... */
 
@@ -173,14 +172,12 @@ doit (void)
   gsasl_free (s1);
   if (res != GSASL_OK)
     {
-      fail ("gsasl_step(4) failed (%d):\n%s\n", res,
-	    gsasl_strerror (res));
+      fail ("gsasl_step(4) failed (%d):\n%s\n", res, gsasl_strerror (res));
       return;
     }
 
   if (debug)
-    printf ("S: %.*s [%c]\n", (int) s2len,
-	    s2, res == GSASL_OK ? 'O' : 'N');
+    printf ("S: %.*s [%c]\n", (int) s2len, s2, res == GSASL_OK ? 'O' : 'N');
 
   /* Let client parse server final... */
 
@@ -188,8 +185,7 @@ doit (void)
   gsasl_free (s2);
   if (res != GSASL_OK)
     {
-      fail ("gsasl_step(5) failed (%d):\n%s\n", res,
-	    gsasl_strerror (res));
+      fail ("gsasl_step(5) failed (%d):\n%s\n", res, gsasl_strerror (res));
       return;
     }
 
@@ -202,6 +198,28 @@ doit (void)
       fail ("Bad authid? %s != %s\n", p, USERNAME);
   }
 
+  {
+    const char *sp =
+      gsasl_property_fast (client, GSASL_SCRAM_SALTED_PASSWORD);
+    if (!sp
+	|| strcmp (sp,
+		   "c4a49510323ab4f952cac1fa99441939"
+		   "e78ea74d6be81ddf7096e87513dc615d") != 0)
+      fail ("client didn't set salted password: %s\n",
+	    gsasl_property_fast (client, GSASL_SCRAM_SALTED_PASSWORD));
+  }
+
+  {
+    const char *sp =
+      gsasl_property_fast (server, GSASL_SCRAM_SALTED_PASSWORD);
+    if (!sp
+	|| strcmp (sp,
+		   "c4a49510323ab4f952cac1fa99441939"
+		   "e78ea74d6be81ddf7096e87513dc615d") != 0)
+      fail ("server didn't set salted password: %s\n",
+	    gsasl_property_fast (client, GSASL_SCRAM_SALTED_PASSWORD));
+  }
+
   if (debug)
     printf ("\n");
 
